@@ -1,17 +1,3 @@
-/*
-  Racket
-  Copyright (c) 2004-2017 PLT Design Inc.
-  Copyright (c) 1995-2001 Matthew Flatt
-  All rights reserved.
-
-  Please see the full copyright in the documentation.
-
-  Originally based on:
-  libscheme
-  Copyright (c) 1994 Brent Benson
-  All rights reserved.
-*/
-
 #ifndef SCHEME_H
 #define SCHEME_H
 
@@ -151,12 +137,6 @@ typedef long FILE;
 # define MSCBOR_IZE(x) MSC_IZE(x)
 #endif
 
-#ifdef SIGSET_IS_SIGNAL
-# define MZ_SIGSET(s, f) signal(s, f)
-#else
-# define MZ_SIGSET(s, f) sigset(s, f)
-#endif
-
 /* C99 allows an array in a struct to be declared
    with [] to indicate that its actual size can be
    any number. The old way was to declare the array
@@ -272,6 +252,12 @@ extern "C"
 {
 #endif
 
+#if defined(MZ_DECLARE_NORETURN) && defined(__GNUC__)
+#define NORETURN __attribute__((__noreturn__))
+#else
+#define NORETURN
+#endif
+
 /* Allowed by all configurations, currently: */
 #define MZ_CAN_ACCESS_THREAD_LOCAL_DIRECTLY
 
@@ -708,13 +694,19 @@ typedef struct Scheme_Offset_Cptr
 #define scheme_make_character(ch) ((((mzchar)ch) < 256) ? scheme_char_constants[(unsigned char)(ch)] : scheme_make_char(ch))
 #define scheme_make_ascii_character(ch) scheme_char_constants[(unsigned char)(ch)]
 
-#define scheme_uchar_find(table, x) (table[(x >> 8) & 0x1FFF][x & 0xFF])
+#define SCHEME_UCHAR_FIND_SHIFT    8
+#define SCHEME_UCHAR_FIND_HI_MASK  0x1FFF
+#define SCHEME_UCHAR_FIND_LO_MASK  0xFF
+
+#define scheme_uchar_find(table, x) (table[(x >> SCHEME_UCHAR_FIND_SHIFT) & SCHEME_UCHAR_FIND_HI_MASK][x & SCHEME_UCHAR_FIND_LO_MASK])
+
+#define SCHEME_ISSPACE_BIT         0x10
 
 #define scheme_isblank(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x1)
 #define scheme_issymbol(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x2)
 #define scheme_ispunc(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x4)
 #define scheme_iscontrol(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x8)
-#define scheme_isspace(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x10)
+#define scheme_isspace(x) ((scheme_uchar_find(scheme_uchar_table, x)) & SCHEME_ISSPACE_BIT)
 /* #define scheme_isSOMETHING(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x20) - not yet used */
 #define scheme_isdigit(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x40)
 #define scheme_isalpha(x) ((scheme_uchar_find(scheme_uchar_table, x)) & 0x80)
@@ -1174,15 +1166,6 @@ typedef struct Scheme_Thread {
 
   struct Scheme_Overflow *overflow;
 
-  struct Scheme_Comp_Env *current_local_env;
-  Scheme_Object *current_local_scope;
-  Scheme_Object *current_local_use_scope;
-  Scheme_Object *current_local_name;
-  Scheme_Object *current_local_modidx;
-  Scheme_Env *current_local_menv;
-  Scheme_Object *current_local_bindings;
-  intptr_t current_phase_shift;
-
   struct Scheme_Marshal_Tables *current_mt;
 
   struct Optimize_Info *constant_folding; /* compiler hack */
@@ -1334,31 +1317,14 @@ enum {
 
   MZCONFIG_INIT_EXN_HANDLER,
 
-  MZCONFIG_EVAL_HANDLER,
-  MZCONFIG_COMPILE_HANDLER,
-  MZCONFIG_LOAD_HANDLER,
-  MZCONFIG_LOAD_COMPILED_HANDLER,
-
   MZCONFIG_PRINT_HANDLER,
   MZCONFIG_PROMPT_READ_HANDLER,
   MZCONFIG_READ_HANDLER,
   MZCONFIG_READ_INPUT_PORT_HANDLER,
 
-  MZCONFIG_READTABLE,
-  MZCONFIG_READER_GUARD,
-
-  MZCONFIG_CAN_READ_GRAPH,
-  MZCONFIG_CAN_READ_COMPILED,
-  MZCONFIG_CAN_READ_BOX,
+  MZCONFIG_CASE_SENS,
   MZCONFIG_CAN_READ_PIPE_QUOTE,
-  MZCONFIG_CAN_READ_DOT,
-  MZCONFIG_CAN_READ_INFIX_DOT,
-  MZCONFIG_CAN_READ_QUASI,
-  MZCONFIG_CAN_READ_READER,
-  MZCONFIG_CAN_READ_LANG,
-  MZCONFIG_READ_DECIMAL_INEXACT,
-  MZCONFIG_READ_CDOT,
-  
+
   MZCONFIG_PRINT_GRAPH,
   MZCONFIG_PRINT_STRUCT,
   MZCONFIG_PRINT_BOX,
@@ -1372,12 +1338,6 @@ enum {
   MZCONFIG_PRINT_LONG_BOOLEAN,
   MZCONFIG_PRINT_AS_QQ,
 
-  MZCONFIG_CASE_SENS,
-  MZCONFIG_SQUARE_BRACKETS_ARE_PARENS,
-  MZCONFIG_CURLY_BRACES_ARE_PARENS,
-  MZCONFIG_SQUARE_BRACKETS_ARE_TAGGED,
-  MZCONFIG_CURLY_BRACES_ARE_TAGGED,
-
   MZCONFIG_ERROR_PRINT_WIDTH,
   MZCONFIG_ERROR_PRINT_CONTEXT_LENGTH,
 
@@ -1389,24 +1349,16 @@ enum {
   MZCONFIG_COMPILE_MODULE_CONSTS,
   MZCONFIG_USE_JIT,
   MZCONFIG_DISALLOW_INLINE,
+  MZCONFIG_COMPILE_TARGET_MACHINE,
 
   MZCONFIG_CUSTODIAN,
   MZCONFIG_INSPECTOR,
   MZCONFIG_CODE_INSPECTOR,
   MZCONFIG_PLUMBER,
 
-  MZCONFIG_USE_COMPILED_KIND,
-  MZCONFIG_USE_COMPILED_ROOTS,
-  MZCONFIG_USE_USER_PATHS,
-  MZCONFIG_USE_LINK_PATHS,
-  MZCONFIG_USE_COMPILED_FILE_CHECK,
-
   MZCONFIG_LOAD_DIRECTORY,
   MZCONFIG_WRITE_DIRECTORY,
 
-  MZCONFIG_COLLECTION_PATHS,
-  MZCONFIG_COLLECTION_LINKS,
-
   MZCONFIG_PORT_PRINT_HANDLER,
 
   MZCONFIG_LOAD_EXTENSION_HANDLER,
@@ -1419,10 +1371,7 @@ enum {
 
   MZCONFIG_RANDOM_STATE,
 
-  MZCONFIG_CURRENT_MODULE_RESOLVER,
-  MZCONFIG_CURRENT_MODULE_NAME,
   MZCONFIG_CURRENT_MODULE_SRC,
-  MZCONFIG_CURRENT_MODULE_LOAD_PATH,
 
   MZCONFIG_ERROR_PRINT_SRCLOC,
 
@@ -1445,8 +1394,6 @@ enum {
   MZCONFIG_LOAD_DELAY_ENABLED,
   MZCONFIG_DELAY_LOAD_INFO,
 
-  MZCONFIG_EXPAND_OBSERVE,
-
   MZCONFIG_LOGGER,
 
   __MZCONFIG_BUILTIN_COUNT__
@@ -1594,13 +1541,6 @@ typedef struct Scheme_Logger Scheme_Logger;
 #define SCHEME_GUARD_FILE_DELETE  0x8
 #define SCHEME_GUARD_FILE_EXISTS  0x10
 
-/*========================================================================*/
-/*                               modules                                  */
-/*========================================================================*/
-
-typedef void (*Scheme_Invoke_Proc)(Scheme_Env *env, intptr_t phase_shift,
-				   Scheme_Object *self_modidx, void *data);
-
 /*========================================================================*/
 /*                               evaluation                               */
 /*========================================================================*/
@@ -1888,6 +1828,7 @@ MZ_EXTERN int scheme_hash_percent_syntax_only; /* Defaults to 0 */
 MZ_EXTERN int scheme_hash_percent_globals_only; /* Defaults to 0 */
 MZ_EXTERN int scheme_binary_mode_stdio; /* Windows-specific; Defaults to 0 */
 MZ_EXTERN int scheme_startup_use_jit; /* Defaults to 1 */
+MZ_EXTERN int scheme_startup_compile_machine_independent; /* Defaults to 0 */
 MZ_EXTERN int scheme_ignore_user_paths; /* Defaults to 0 */
 MZ_EXTERN int scheme_ignore_link_paths; /* Defaults to 0 */
 
@@ -1895,12 +1836,15 @@ MZ_EXTERN void scheme_set_case_sensitive(int);
 MZ_EXTERN void scheme_set_allow_set_undefined(int);
 MZ_EXTERN void scheme_set_binary_mode_stdio(int);
 MZ_EXTERN void scheme_set_startup_use_jit(int);
+MZ_EXTERN void scheme_set_startup_compile_machine_independent(int);
 MZ_EXTERN void scheme_set_startup_load_on_demand(int);
 MZ_EXTERN void scheme_set_ignore_user_paths(int);
 MZ_EXTERN void scheme_set_ignore_link_paths(int);
 MZ_EXTERN void scheme_set_cross_compile_mode(int);
 MZ_EXTERN void scheme_set_logging(int syslog_level, int stderr_level);
 MZ_EXTERN void scheme_set_logging_spec(Scheme_Object *syslog_level, Scheme_Object *stderr_level);
+MZ_EXTERN void scheme_set_logging2(int syslog_level, int stderr_level, int stdout_level);
+MZ_EXTERN void scheme_set_logging2_spec(Scheme_Object *syslog_level, Scheme_Object *stderr_level, Scheme_Object *stdout_level);
 
 MZ_EXTERN int scheme_get_allow_set_undefined();
 
@@ -1934,11 +1878,11 @@ MZ_EXTERN void scheme_set_console_printf(scheme_console_printf_t p);
 typedef void (*scheme_console_output_t)(char *str, intptr_t len);
 MZ_EXTERN scheme_console_output_t scheme_console_output;
 MZ_EXTERN void scheme_set_console_output(scheme_console_output_t p);
+MZ_EXTERN void scheme_ensure_console_ready();
 MZ_EXTERN void (*scheme_sleep)(float seconds, void *fds);
 MZ_EXTERN void (*scheme_notify_multithread)(int on);
 MZ_EXTERN void (*scheme_wakeup_on_input)(void *fds);
 MZ_EXTERN int (*scheme_check_for_break)(void);
-MZ_EXTERN Scheme_Object *(*scheme_module_demand_hook)(int c, Scheme_Object **a);
 #ifdef MZ_PRECISE_GC
 MZ_EXTERN void *(*scheme_get_external_stack_val)(void);
 MZ_EXTERN void (*scheme_set_external_stack_val)(void *);
@@ -1973,6 +1917,9 @@ MZ_EXTERN void scheme_set_compiled_file_paths(Scheme_Object *list);
 MZ_EXTERN void scheme_set_compiled_file_roots(Scheme_Object *list);
 #ifdef DOS_FILE_SYSTEM
 MZ_EXTERN void scheme_set_dll_path(wchar_t *s);
+typedef void *(*scheme_dll_open_proc)(const char *name, int as_global);
+typedef void *(*scheme_dll_find_object_proc)(void *h, const char *name);
+MZ_EXTERN void scheme_set_dll_procs(scheme_dll_open_proc, scheme_dll_find_object_proc);
 #endif
 
 MZ_EXTERN void scheme_init_collection_paths(Scheme_Env *global_env, Scheme_Object *extra_dirs);
@@ -2035,6 +1982,12 @@ MZ_EXTERN Scheme_Object *scheme_register_parameter(Scheme_Prim *function, char *
 
 #endif /* SCHEME_DIRECT_EMBEDDED */
 
+/*========================================================================*/
+/*                             OS signals                                 */
+/*========================================================================*/
+
+typedef void (*Scheme_Signal_Handler_Proc)(int);
+
 /*========================================================================*/
 /*                              addrinfo                                  */
 /*========================================================================*/
